# cluster_setting.py
# definition and properties of a dipole-cluster
# Ingo Rehberg, Oct 28 2018
import numpy as np
class Cluster:
    def __init__(self):                                        # init functions for all constants needed
        self.x, self.y, self.z, self.u,self.v,self.w = np.loadtxt(
                "animation/config.dat",dtype='double',unpack=True, skiprows=1,usecols=(0,1,2,3,4,5))    #

#        self.x, self.y, self.z = np.meshgrid(    np.linspace(-1, 1, 2),            # 8 Positions with spacing 2
#                                    np.linspace(-1, 1, 2),
#                                    np.linspace(-1, 1, 2))
#        self.set_ang(tau)
        print ("u=",self.u)
        print ("v=",self.v)
        print ("w=",self.w)
        self.num=len(self.x)

    def get_xyzuvw(self):
        #self.x, self.y, self.z, self.u,self.v,self.w = np.loadtxt("config.dat",dtype='double',unpack=True, usecols=(0,1,2,3,4,5))
        #print("u_here=", self.u)
        return np.array(self.x),np.array(self.y),np.array(self.z),np.array(self.u),np.array(self.v),np.array(self.w)
    """
    def set_ang(self,tau):                                            # Make the direction data for the arrows
        self.t=np.pi/180. *tau                                        # in radian, alpha her =tau in paper
        self.u = -  np.sin(self.t  - 4./3.*np.pi)     * np.sqrt(2./3.) * self.y*self.z    # *self.y*self.z for symmetry
        self.v = -  np.sin(self.t -  2./3.*np.pi)     * np.sqrt(2./3.) * self.x*self.z     # 2./3.*np.pi vector components phase shifted by 120 degrees
        self.w = - np.sin(self.t               )    * np.sqrt(2./3.) * self.x*self.y
        return
    """
    def energy(self,px,py,pz):                                            # calculate the field energy at position px,py,pz
        lbx=0.
        lby=0.
        lbz=0.
        for i in range(self.num):
            rx=px-self.x[i]
            ry=py-self.y[i]
            rz=pz-self.z[i]
            r2=pow(rx,2)+pow(ry,2)+pow(rz,2)
            scpr=self.u[i]*rx+self.v[i]*ry+self.w[i]*rz
            lbx+=(3.*scpr*rx - r2*self.u[i])/pow(r2,2.5)
            lby+=(3.*scpr*ry - r2*self.v[i])/pow(r2,2.5)
            lbz+=(3.*scpr*rz - r2*self.w[i])/pow(r2,2.5)
        return np.sqrt(lbx*lbx+lby*lby+lbz*lbz)

    def radial_flux(self,px,py,pz):                                            # calculate the field energy at position px,py,pz
        lbx=0.
        lby=0.
        lbz=0.
        for i in range(self.num):
            rx=px-self.x[i]
            ry=py-self.y[i]
            rz=pz-self.z[i]
            r2=pow(rx,2)+pow(ry,2)+pow(rz,2)
            scpr=self.u[i]*rx+self.v[i]*ry+self.w[i]*rz
            lbx+=(3.*scpr*rx - r2*self.u[i])/pow(r2,2.5)
            lby+=(3.*scpr*ry - r2*self.v[i])/pow(r2,2.5)
            lbz+=(3.*scpr*rz - r2*self.w[i])/pow(r2,2.5)
        return (lbx*px+lby*py+lbz*pz)/np.sqrt(px*px+py*py+pz*pz)

    def potential(self,px,py,pz):                                            # calculate the potential at position px,py,pz
        pot=0.
        for i in range(self.num):
            rx=px-self.x[i]
            ry=py-self.y[i]
            rz=pz-self.z[i]
            r2=pow(rx,2)+pow(ry,2)+pow(rz,2)
            scpr=self.u[i]*rx+self.v[i]*ry+self.w[i]*rz
            pot+=scpr/pow(r2,1.5)
        return pot

    def ind_x(self,px,py,pz,direction):                    # calculate the field component along dir at position px,py,pz
        lbx=0.
        lby=0.
        lbz=0.
        for i in range(self.num):
            rx=px-self.x[i]
            ry=py-self.y[i]
            rz=pz-self.z[i]
            r2=pow(rx,2)+pow(ry,2)+pow(rz,2)
            scpr=self.u[i]*rx+self.v[i]*ry+self.w[i]*rz #scpr= scalar product
            lbx+=(3.*scpr*rx - r2*self.u[i])/pow(r2,2.5)
            lby+=(3.*scpr*ry - r2*self.v[i])/pow(r2,2.5)
            lbz+=(3.*scpr*rz - r2*self.w[i])/pow(r2,2.5)
        return (lbx*direction[0]+lby*direction[1]+lbz*direction[2])

    def b_feld(self,px,py,pz):                                # calculate the field components at position px,py,pz
        lbx=0.
        lby=0.
        lbz=0.
        for i in range(self.num):
            rx=px-self.x[i]
            ry=py-self.y[i]
            rz=pz-self.z[i]
            r2=pow(rx,2)+pow(ry,2)+pow(rz,2)
            scpr=self.u[i]*rx+self.v[i]*ry+self.w[i]*rz
            lbx+=(3.*scpr*rx - r2*self.u[i])/pow(r2,2.5)
            lby+=(3.*scpr*ry - r2*self.v[i])/pow(r2,2.5)
            lbz+=(3.*scpr*rz - r2*self.w[i])/pow(r2,2.5)
        return (lbx,lby,lbz)
    """
    def analytic(self,px):                                # analytic expression for the asymptote
        return 105.*np.sqrt(3./2.)*4.*pow(px*np.sqrt(2.),-7)*np.sin(self.t)
    """