"""
dipole_cube_examination_geometry.py
Ingo Rehberg, Feb 03, 2020
"""
import numpy as np


class Geo:
    def __init__(self, richtung=np.array([1., 1., 0.]),
                 origin=np.array([0, 0, 0]), length=20., d_p=2.,
                 width=2, st=0.1):  # line and perpendicular plane
        self.px = self.py = self.pz = np.arange(0, length, length/100.)
        # position vectors x,y,z
        self.iii = int(2.*width/st+1.1)  # resolution of the plane surfaces
        self.X = self.Y = self.Z = self.XPL = self.YPL = np.zeros(
            (self.iii, self.iii))  # coordinates for plane surfaces, 3d and 2d
        self.richtung = richtung
        self.origin = origin
        self.length = length
        self.d_p = d_p
        self.width = width
        self.st = st
        self.new_geo()

    def new_geo(self):
        self.direction = self.richtung/np.sqrt(np.dot(self.richtung,
                                                      self.richtung))
        self.px = np.arange(0, self.length,
                            self.length/100.)*self.direction[0]+self.origin[0]
        self.py = np.arange(0, self.length,
                            self.length/100.)*self.direction[1]+self.origin[1]
        self.pz = np.arange(0, self.length,
                            self.length/100.)*self.direction[2]+self.origin[2]
        self.ps = np.sqrt(pow(self.px-self.origin[0], 2) +
                          pow(self.py-self.origin[1], 2) +
                          pow(self.pz-self.origin[2], 2))  # distance from orig
        self.cdist = np.sqrt(pow(self.px, 2) +
                             pow(self.py, 2) +
                             pow(self.pz, 2))  # distance from center
        # generate a plane perpendicular to "direction", within the d_p:
        self.d_cross_z = (-self.direction[1], self.direction[0], 0)/np.sqrt(
            self.direction[1]*self.direction[1] +
            self.direction[0]*self.direction[0])
        self.perp = np.cross(self.direction, self.d_cross_z)
        self.X = np.zeros((self.iii, self.iii))  # set to zero
        self.Y = np.zeros((self.iii, self.iii))  # before accumulation
        self.Z = np.zeros((self.iii, self.iii))  # in double loop below
        self.XPL, self.YPL = np.meshgrid(
            np.arange(-self.width, self.width+self.st/2., self.st),
            np.arange(-self.width, self.width+self.st/2., self.st))
        for i in range(self.iii):
            fi = -self.width+i*self.st
            for j in range(self.iii):
                fj = -self.width+j*self.st
                self.X[j, i] += (fi*self.d_cross_z[0] +
                                 fj*self.perp[0] +
                                 self.direction[0]*self.d_p)
                self.Y[j, i] += (fi*self.d_cross_z[1] +
                                 fj*self.perp[1] +
                                 self.direction[1]*self.d_p)
                self.Z[j, i] += (fi*self.d_cross_z[2] +
                                 fj*self.perp[2] +
                                 self.direction[2]*self.d_p)
