"""
dipole_cube_figures: Ingo Rehberg, Feb 03, 2020
"""
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import cm


def bild1(ax1, g, mycube, display_plane, display_sphere,
          display_diag, display_edge, trp, sphere_resol):
    ax1.cla()
    ax1.xaxis.set_major_locator(plt.MultipleLocator(0.5))
    ax1.yaxis.set_major_locator(plt.MultipleLocator(0.5))
    ax1.zaxis.set_major_locator(plt.MultipleLocator(0.5))
    ax1.set_xlim(-1.48, 1.48)
    ax1.set_ylim(-1.48, 1.48)
    ax1.set_zlim(-1.48, 1.48)
    ax1.set_xlabel("x")
    ax1.set_ylabel("y")
    ax1.set_zlabel("z")
    ax1.quiver(g.origin[0], g.origin[1], g.origin[2],
               g.direction[0], g.direction[1], g.direction[2],
               length=1, arrow_length_ratio=0.1, normalize=False, color='gray')
    x, y, z, u, v, w = mycube.get_xyzuvw()

    ax1.quiver(x, y, z, u, v, w, length=1.5, pivot='middle',
               arrow_length_ratio=0.5, normalize=False, color='r', linewidth=4)
    ax1.scatter(x, y, z, color='black', marker='o', s=100)
    if display_edge:                # plot the edges of the cube
        for i in range(2):
            for j in range(2):
                ax1.plot((x[i][j][0], x[i][j][1]),
                         (y[i][j][0], y[i][j][1]),
                         (z[i][i][0], z[i][j][1]), c='b', ls="--")
                ax1.plot((x[i][0][j], x[i][1][j]),
                         (y[i][0][j], y[i][1][j]),
                         (z[i][0][j], z[i][1][j]), c='b', ls="--")
                ax1.plot((x[0][i][j], x[1][i][j]),
                         (y[0][i][j], y[1][i][j]),
                         (z[0][i][j], z[1][i][j]), c='b', ls="--")
    if display_diag:  # show the space diagonals (rotation axis of dipoles)
        ax1.plot((x[0][0][0], x[1][1][1]), (y[0][0][0], y[1][1][1]),
                 (z[0][0][0], z[1][1][1]), c='g', ls="--")
        ax1.plot((x[0][1][0], x[1][0][1]), (y[0][1][0], y[1][0][1]),
                 (z[0][1][0], z[1][0][1]), c='g', ls="--")
        ax1.plot((x[0][0][1], x[1][1][0]), (y[0][0][1], y[1][1][0]),
                 (z[0][0][1], z[1][1][0]), c='g', ls="--")
        ax1.plot((x[0][1][1], x[1][0][0]), (y[0][1][1], y[1][0][0]),
                 (z[0][1][1], z[1][0][0]), c='g', ls="--")
    if display_plane:  # show one plane perpendicular to direction
        ens = mycube.ind_x(g.X, g.Y, g.Z, g.direction)  # magnetic induction on
        # surface X,Y,Z along the "direction"
        ens -= ens.min()
        ens /= ens.max()
        plane_colors = np.empty(g.X.shape)  # the size of the color map
        plane_colors = plt.cm.bwr(ens)  # the type of color coding
        plane_colors[:, :, 3] = trp  # transparency
        ax1.plot_surface(g.X, g.Y, g.Z, facecolors=plane_colors,
                         linewidth=5.0, shade=False)
        ax1.plot(g.X[0, :], g.Y[0, :], g.Z[0, :], color='gray', linewidth=8)
        ax1.plot(g.X[g.iii-1, :], g.Y[g.iii-1, :], g.Z[g.iii-1, :],
                 color='gray', linewidth=8)
        ax1.plot(g.X[:, 0], g.Y[:, 0], g.Z[:, 0], color='gray', linewidth=8)
        ax1.plot(g.X[:, g.iii-1], g.Y[:, g.iii-1], g.Z[:, g.iii-1],
                 color='gray', linewidth=8)
    if display_sphere:  # show the field on sherical surface around cuboid
        radius = 2.5  # radius of the sphere
        us = np.linspace(0, 2 * np.pi, sphere_resol)
        vs = np.linspace(0, np.pi, sphere_resol/2)
        xs = radius * np.outer(np.cos(us), np.sin(vs))
        ys = radius * np.outer(np.sin(us), np.sin(vs))
        zs = radius * np.outer(np.ones(np.size(us)), np.cos(vs))
        ens = mycube.radial_flux(xs, ys, zs)  # flux comp. radial direction
        ens -= ens.min()
        ens /= ens.max()
        mycolors = np.empty(xs.shape)  # the size of the color map
        mycolors = plt.cm.bwr(ens)  # the type of color coding
        mycolors[:, :, 3] = trp
        ax1.plot_surface(xs, ys, zs, facecolors=mycolors,
                         rcount=sphere_resol, ccount=sphere_resol/2)


def bild2(ax2, g, mycube, loglog):
    ax2.cla()
    ax2.set_ylabel(r"$B_\mathrm{x,y,z}$")
    ax2.set_xlabel(
        r"Position from (%3.1f,%3.1f,%3.1f) to (%3.1f,%3.1f,%3.1f)$\,L$"
        % (g.origin[0], g.origin[1], g.origin[2],
           g.direction[0], g.direction[1], g.direction[2]))
    bx, by, bz = mycube.b_feld(g.px, g.py, g.pz)  # get fields along the line
    if loglog:
        supermax = 1.05*max(max(bx), max(by), max(bz), -min(bx),
                            -min(by), -min(bz))
        ax2.set_ylim(supermax*1.e-7, supermax)
        l1, = ax2.loglog(g.ps, abs(bx), color="red", label=r"$B_\mathrm{x}$")
        l2, = ax2.loglog(g.ps, abs(by), color="green",
                         linestyle='dashed', label=r"$B_\mathrm{y}$")
        l3, = ax2.loglog(g.ps, abs(bz), color="blue", label=r"$B_\mathrm{z}$")
        l4, = ax2.loglog(g.ps[g.iii//16:],
                         abs(mycube.analytic(g.px[g.iii//16:])),
                         color="black", linestyle='dashed',
                         label=r"$B_\mathrm{a,(1,1,0)}$")
        legend = ax2.legend(handles=[l1, l2, l3, l4], loc='best',
                            shadow=True, fontsize='small')
        legend.get_frame().set_facecolor('#0F00000F')
    else:
        supermax = max(max(bx), max(by), max(bz))
        supermin = min(min(bx), min(by), min(bz))
        dx = (supermax-supermin)/20
        ax2.set_ylim(supermin-dx, supermax+dx)
        l1, = ax2.plot(g.ps, bx, color="red", label=r"$B_\mathrm{x}$")
        l2, = ax2.plot(g.ps, by, color="green",
                       linestyle='dashed', label=r"$B_\mathrm{y}$")
        l3, = ax2.plot(g.ps, bz, color="blue", label=r"$B_\mathrm{z}$")
        l4, = ax2.plot(g.ps[g.iii//16:], mycube.analytic(g.px[g.iii//16:]),
                       color="black", linestyle='dashed',
                       label=r"$B_\mathrm{a,(1,1,0)}$")
        legend = ax2.legend(handles=[l1, l2, l3, l4], loc='best',
                            shadow=True, fontsize='small')
        legend.get_frame().set_facecolor('#0F00000F')


def bild3(ax3, g, mycube):
    ax3.cla()
    ax3.text(0, g.width*1.05, 'Energy',
             horizontalalignment='center', verticalalignment='bottom',
             rotation=0)
    ax3.set_ylabel(
        "upwards (%3.1f,%3.1f,%3.1f)" % (g.perp[0], g.perp[1], g.perp[2]))
    H3 = mycube.energy(g.X, g.Y, g.Z)
    ax3.contour(g.XPL, g.YPL, H3, cmap=cm.gray)
    ax3.imshow(H3, interpolation='bilinear', origin='lower',
               cmap=cm.bwr, extent=(-g.width, g.width, -g.width, g.width))


def bild4(ax4, g, mycube):
    ax4.cla()
    ax4.text(0, g.width*1.05, 'Potential', horizontalalignment='center',
             verticalalignment='bottom', rotation=0)
    H4 = mycube.potential(g.X, g.Y, g.Z)
    ax4.contour(g.XPL, g.YPL, H4, cmap=cm.gray)
    ax4.imshow(H4, interpolation='bilinear', origin='lower',
               cmap=cm.bwr, extent=(-g.width, g.width, -g.width, g.width))


def bild5(ax5, g, mycube):
    ax5.cla()
    ax5.text(g.width, 0, r'$B_\mathrm{perp}$', horizontalalignment='left',
             verticalalignment='center')
    H5 = mycube.ind_x(g.X, g.Y, g.Z, g.direction)
    ax5.contour(g.XPL, g.YPL, H5, cmap=cm.gray)
    ax5.imshow(H5, interpolation='bilinear', origin='lower',
               cmap=cm.bwr, extent=(-g.width, g.width, -g.width, g.width))


def bild6(ax6, g, mycube):
    ax6.cla()
    ax6.text(g.width, 0, r'$B_\mathrm{up}$', horizontalalignment='left',
             verticalalignment='center', rotation=0)
    H6 = mycube.ind_x(g.X, g.Y, g.Z, g.perp)
    ax6.contour(g.XPL, g.YPL, H6, cmap=cm.gray)
    ax6.imshow(H6, interpolation='bilinear', origin='lower',
               cmap=cm.bwr, extent=(-g.width, g.width, -g.width, g.width))


def bild7(ax7, g, mycube):
    ax7.cla()
    ax7.text(g.width, 0, r'$B_\mathrm{hor}$', horizontalalignment='left',
             verticalalignment='center', rotation=0)
    ax7.set_xlabel(
        "horizontal (%3.1f,%3.1f,%3.1f)" % (g.d_cross_z[0],
                                            g.d_cross_z[1],
                                            g.d_cross_z[2]))
    H7 = mycube.ind_x(g.X, g.Y, g.Z, g.d_cross_z)
    ax7.contour(g.XPL, g.YPL, H7, cmap=cm.gray)
    ax7.imshow(H7, interpolation='bilinear', origin='lower',
               cmap=cm.bwr, extent=(-g.width, g.width, -g.width, g.width))
